import { createReducer } from '../utils/reducer'
import { REQUEST_OK, REQUEST_KO, LOGOUT } from '../constants/ActionTypes'
import { setToken } from '../utils/auth'
import { has, isEqual } from 'lodash'

const initialState = {
    isAuthenticated: false
}

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
const checkAuthentication = (isAuthenticated, action) => {
    if (!has(action.payload, 'status')) {
        return isAuthenticated
    }

    // success auth
    if (isEqual(action.payload.status, 200)) {
        if (has(action.payload, 'token')) {
            setToken(action.payload.token)

            return true;
        }

        return true
    }

    // error auth
    if (isEqual(action.payload.status, 401)) {
        return false
    }

    return isAuthenticated
}

/**
 * 
 * @param {*} isAuthenticated 
 * @param {*} action 
 */
const revokeAuthentication = (isAuthenticated, action) => {
    setToken(null)
    return false
}

/**
 * 
 */
const authenticationCheckReducer = createReducer(initialState, {
    [REQUEST_OK]: checkAuthentication,
    [REQUEST_KO]: checkAuthentication,
    [LOGOUT]: revokeAuthentication

})

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
export const auth = (state = initialState, action) => {
    return {
        isAuthenticated: authenticationCheckReducer(state.isAuthenticated, action)
    }
}